local L = LibStub("AceLocale-3.0"):GetLocale("QuestItem")
local media = LibStub:GetLibrary("LibSharedMedia-3.0");

QuestItem.BagBackdrop = {
	bgFile = "Interface\\AddOns\\QuestItem\\Textures\\BagFrameBackground",
	edgeFile = "Interface\\AddOns\\QuestItem\\Textures\\BagFrameBorder",
	tile = true,
	edgeSize = 32,
	tileSize = 32,
	insets = {left = 7, right = 6, top = 7, bottom = 6}
}

----------------------
-- Frame Routines
----------------------

function QuestItem:CreateBagFrame()
	if not self.BagFrame then
		self.BagFrame = CreateFrame("Frame", "QuestItemBag", UIParent)
	end
	self.BagFrame:SetToplevel(true)
	self.BagFrame:EnableMouse(true)
	self.BagFrame:SetMovable(true)
	self.BagFrame:SetFrameStrata("MEDIUM")
	self.BagFrame:SetFrameLevel(2)
	self.BagFrame:SetWidth(185)
	self.BagFrame:RegisterForDrag("LeftButton")
	self.BagFrame:SetPoint(self.db.char.point, self.db.char.relativeTo, self.db.char.relativePoint, self.db.char.xOffset, self.db.char.yOffset)
	self.BagFrame:SetScript("OnDragStart", 
		function()
			self.BagFrame:StartMoving();
		end );
	self.BagFrame:SetScript("OnDragStop",
		function()
			self.BagFrame:StopMovingOrSizing();
			self.BagFrame:SetScript("OnUpdate", nil);
			self:FinishedMoving(self.db.char, self.BagFrame);
		end );
	self.BagFrame:SetScript("OnEvent", function() QuestItem:QuestBag_OnEvent() end)
	self.BagFrame:SetScript("OnHide", function() QuestItem:QuestBag_OnHide() end)
	
	self.BagFrame.textureContainer = CreateFrame("Frame", nil, self.BagFrame)
	self.BagFrame.textureContainer:SetBackdrop(self.BagBackdrop)
	self.BagFrame.textureContainer:SetBackdropColor(1,1,1,1)
	self.BagFrame.textureContainer:SetPoint("TOPLEFT", self.BagFrame, "TOPLEFT")
	self.BagFrame.textureContainer:SetPoint("BOTTOMRIGHT", self.BagFrame, "BOTTOMRIGHT")
	self.BagFrame.textureContainer:SetAlpha(1)
	self.BagFrame.textureContainer:Show()
	self.BagFrame.portrait = self.BagFrame.textureContainer:CreateTexture(nil, "ARTWORK")
	self.BagFrame.portrait:SetWidth(34)
	self.BagFrame.portrait:SetHeight(34)
	self.BagFrame.portrait:SetPoint("TOPLEFT", self.BagFrame, "TOPLEFT", 3, -5)
	self.BagFrame.portrait:Show()
	local portraitFrame = self.BagFrame.textureContainer:CreateTexture(nil, "OVERLAY")
 	portraitFrame:SetTexture("Interface\\AddOns\\QuestItem\\Textures\\BagFramePortraitFrame")
	portraitFrame:SetTexCoord(0, 0.745, 0, 0.745)
	portraitFrame:SetWidth(48)
	portraitFrame:SetHeight(48)
	portraitFrame:SetPoint("TOPLEFT", self.BagFrame, "TOPLEFT", -4, 0)
	portraitFrame:Show()
	self.BagFrame.name = self.BagFrame.textureContainer:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
	self.BagFrame.name:SetWidth(112)
	self.BagFrame.name:SetHeight(12)
	self.BagFrame.name:SetPoint("TOPRIGHT", self.BagFrame, "TOPRIGHT", -33, -9)
	self.BagFrame.name:SetText("Quest Item Bag")
	local closeButton = CreateFrame("Button", nil, self.BagFrame.textureContainer, "UIPanelCloseButton")
	closeButton:SetScript("OnClick", function() self.BagFrame:Hide() end)
	closeButton:SetPoint("TOPRIGHT", self.BagFrame, "TOPRIGHT", 0, -1)
--~ 	table.insert(UISpecialFrames, "QuestItemBag")
end

function QuestItem:UpdateBagFrame(bagID)
	-- check all items in bags and add to quest bag frame 
	if bagID then
		self:Print("updating bag frame "..bagID)
		self:ClearQuestBag()
		for bag = 0,4 do
			for slot = 1, GetContainerNumSlots(bag) do
				local itemLink = GetContainerItemLink(bag, slot)
				if itemLink then -- check if its a quest item
					local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(itemLink)
					if itemType == L["Quest"] then
						self:Print(itemName.." found in bag - its a quest item")
						self:AddItemtoQuestBag(bag, slot, itemLink)
					end
				end
			end
		end
		self:RedrawBagFrame()
	end
end

function QuestItem:RedrawBagFrame()
	local rows = math.floor(self.db.char.questbag.count / 4) + 1 -- extra row to ensure always space displayed ie: empty bag
	self.BagFrame:SetHeight((rows + 1)*40 + 16)

	for i = 1, self.db.char.questbag.count do
		local itemButton = self:SetupItemButton(i, self.db.char.questbag.count)
		local questitem = self.db.char.questbag[i]
		local texture, itemCount, locked, quality, readable = GetContainerItemInfo(questitem.bag, questitem.slot)
		SetItemButtonCount(itemButton, itemCount)
		SetItemButtonDesaturated(itemButton, locked, 0.5, 0.5, 0.5)
		itemButton.readable = readable
		if texture then
			SetItemButtonTexture(itemButton, texture)
		else
			SetItemButtonTexture(itemButton, "Interface\\AddOns\\QuestItem\\Textures\\EmptySlot")
		end
		SetItemButtonNormalTextureVertexColor(itemButton, 1, 1, 1)
		itemButton.questitem = questitem
		itemButton:Show()
	end
	for i = self.db.char.questbag.count + 1, rows * 4 do
		local itemButton = self:SetupItemButton(i, rows * 4)
		SetItemButtonTexture(itemButton, "Interface\\AddOns\\QuestItem\\Textures\\EmptySlot")
		SetItemButtonNormalTextureVertexColor(itemButton, 1, 1, 1)
		itemButton.questitem = nil 
		itemButton:Show()
	end
	self.BagFrame:Show()
end

function QuestItem:SetupItemButton(index, max)
	local name = self.BagFrame:GetName()
	local buttonName = name.."Item"..index
	local itemButton = getglobal(buttonName)
	if not itemButton then
		self:Print("creating button "..index.." name="..buttonName)
		itemButton = CreateFrame("Button", buttonName, self.BagFrame, "ContainerFrameItemButtonTemplate")
		self:HookScript(itemButton, "OnEnter", "ItemButton_OnEnter")
--~ 		self:HookScript(itemButton, "OnEvent", "ItemButton_OnEvent")
--~ 		self:HookScript(itemButton, "PreClick", "ItemButton_PreClick")
--~ 		self:HookScript(itemButton, "PostClick", "ItemButton_PostClick")
	end
	itemButton:ClearAllPoints()
	if (index == 1) then
		-- put the buttom bottom right
		itemButton:SetPoint("BOTTOMRIGHT", self.BagFrame, "BOTTOMRIGHT", -13, 13)
	elseif (mod((index-1), 4) == 0) then
		itemButton:SetPoint("BOTTOMRIGHT", name.."Item"..(index - 4), "TOPRIGHT", 0, 3)
	else
		itemButton:SetPoint("BOTTOMRIGHT", name.."Item"..(index - 1), "BOTTOMLEFT", -3, 0)
	end
	return itemButton
end

function QuestItem:ItemButton_OnEnter(button, ...)
	if button and button.questitem then
		self:Print("trying to show tooltip for bag:"..button.questitem.bag.." slot "..button.questitem.slot)
		if (button:GetRight() >= (GetScreenWidth()/2)) then
			GameTooltip:SetOwner(button, "ANCHOR_LEFT")
		else
			GameTooltip:SetOwner(button, "ANCHOR_RIGHT")
		end
		GameTooltip:SetBagItem(button.questitem.bag, button.questitem.slot)
		CursorUpdate()
	end
end
